"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _f2Graphic = require("@antv/f2-graphic");

var _util = require("@antv/util");

var _component = _interopRequireDefault(require("../base/component"));

var _timeline = _interopRequireDefault(require("../timeline"));

var _layout = _interopRequireDefault(require("../base/layout"));

var _equal = _interopRequireDefault(require("../base/equal"));

var _animation = _interopRequireDefault(require("./animation"));

var _util2 = require("../util");

var _updater = require("./updater");

var _theme = _interopRequireDefault(require("../theme"));

var _diff = require("../base/diff");

function measureText(canvas, px2hd) {
  return function (text, font) {
    var fontSize = font.fontSize,
        fontFamily = font.fontFamily,
        fontStyle = font.fontStyle,
        fontWeight = font.fontWeight,
        fontVariant = font.fontVariant;
    var shape = canvas.addShape('text', {
      attrs: {
        x: 0,
        y: 0,
        fontSize: px2hd(fontSize),
        fontFamily: fontFamily,
        fontStyle: fontStyle,
        fontWeight: fontWeight,
        fontVariant: fontVariant,
        text: text
      }
    });

    var _shape$getBBox = shape.getBBox(),
        width = _shape$getBBox.width,
        height = _shape$getBBox.height;

    shape.remove(true);
    return {
      width: width,
      height: height
    };
  };
} // 顶层Canvas标签


var Canvas = /*#__PURE__*/function (_Component) {
  (0, _inherits2.default)(Canvas, _Component);

  var _super = (0, _createSuper2.default)(Canvas);

  function Canvas(props) {
    var _this;

    (0, _classCallCheck2.default)(this, Canvas);
    _this = _super.call(this, props);
    var context = props.context,
        pixelRatio = props.pixelRatio,
        width = props.width,
        height = props.height,
        _props$animate = props.animate,
        animate = _props$animate === void 0 ? true : _props$animate,
        _props$px2hd = props.px2hd,
        px2hd = _props$px2hd === void 0 ? _util2.px2hd : _props$px2hd,
        customTheme = props.theme,
        customStyle = props.style; // 创建G的canvas

    var canvas = (0, _f2Graphic.createCanvas)({
      context: context,
      pixelRatio: pixelRatio,
      width: width,
      height: height
    });
    var _canvas$_attrs = canvas._attrs,
        canvasWidth = _canvas$_attrs.width,
        canvasHeight = _canvas$_attrs.height;
    var theme = px2hd((0, _util.deepMix)({}, _theme.default, customTheme));
    var style = px2hd((0, _objectSpread2.default)({
      left: 0,
      top: 0,
      width: canvasWidth,
      height: canvasHeight,
      padding: theme.padding
    }, customStyle));

    var layout = _layout.default.fromStyle(style); // 组件更新器


    var updater = (0, _updater.createUpdater)((0, _assertThisInitialized2.default)(_this)); // 供全局使用的一些变量

    var componentContext = {
      root: (0, _assertThisInitialized2.default)(_this),
      canvas: canvas,
      left: layout.left,
      top: layout.top,
      width: layout.width,
      height: layout.height,
      theme: theme,
      px2hd: px2hd,
      measureText: measureText(canvas, px2hd)
    }; // 动画模块

    var animation = new _animation.default(canvas);
    _this.canvas = canvas;
    _this.container = canvas;
    _this.layout = layout;
    _this.context = componentContext;
    _this.updater = updater;
    _this.animate = animate;
    _this.animation = animation;
    _this.theme = theme;
    return _this;
  }

  (0, _createClass2.default)(Canvas, [{
    key: "renderComponents",
    value: function renderComponents(components) {
      if (!components || !components.length) {
        return;
      }

      (0, _diff.renderComponent)(components);
      this.draw();
    }
  }, {
    key: "update",
    value: function update(nextProps) {
      var props = this.props;

      if ((0, _equal.default)(nextProps, props)) {
        return;
      }

      this.props = nextProps;
      this.render();
    }
  }, {
    key: "draw",
    value: function draw() {
      var canvas = this.canvas,
          animate = this.animate,
          animation = this.animation,
          props = this.props,
          children = this.children;

      if (animate === false) {
        canvas.draw();
        return;
      } // 查找timeline


      var timeline = _timeline.default.find(children);

      var onAnimationEnd = props.onAnimationEnd; // 执行动画

      animation.abort();
      animation.play(canvas, function () {
        if (!timeline || !timeline.next()) {
          onAnimationEnd && onAnimationEnd();
        }
      });
    }
  }, {
    key: "render",
    value: function render() {
      var lastChildren = this.children,
          props = this.props;
      var nextChildren = props.children;
      (0, _diff.renderChildren)(this, nextChildren, lastChildren);
      this.draw();
      return null;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var canvas = this.canvas;
      canvas.destroy();
    }
  }]);
  return Canvas;
}(_component.default);

var _default = Canvas;
exports.default = _default;