"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _component = _interopRequireDefault(require("../../base/component"));

var _jsx = require("../../jsx");

// view 的默认配置
var defaultStyle = {
  showTitle: false,
  showCrosshairs: false,
  crosshairsType: 'y',
  crosshairsStyle: {
    stroke: 'rgba(0, 0, 0, 0.25)',
    lineWidth: '2px'
  },
  showTooltipMarker: true,
  background: {
    radius: '2px',
    fill: 'rgba(0, 0, 0, 0.65)',
    padding: ['6px', '10px']
  },
  titleStyle: {
    fontSize: '24px',
    fill: '#fff',
    textAlign: 'start',
    textBaseline: 'top'
  },
  nameStyle: {
    fontSize: '24px',
    fill: 'rgba(255, 255, 255, 0.65)',
    textAlign: 'start',
    textBaseline: 'middle'
  },
  valueStyle: {
    fontSize: '24px',
    fill: '#fff',
    textAlign: 'start',
    textBaseline: 'middle'
  },
  joinString: ': ',
  showItemMarker: true,
  itemMarkerStyle: {
    width: '12px',
    radius: '6px',
    symbol: 'circle',
    lineWidth: '2px',
    stroke: '#fff'
  },
  layout: 'horizontal',
  snap: false
};

function directionEnabled(mode, dir) {
  if (mode === undefined) {
    return true;
  } else if (typeof mode === 'string') {
    return mode.indexOf(dir) !== -1;
  }

  return false;
}

var TooltipView = /*#__PURE__*/function (_Component) {
  (0, _inherits2.default)(TooltipView, _Component);

  var _super = (0, _createSuper2.default)(TooltipView);

  function TooltipView(props) {
    var _this;

    (0, _classCallCheck2.default)(this, TooltipView);
    _this = _super.call(this, props);
    _this.rootRef = {
      current: null
    };
    _this.itemsRef = {
      current: null
    };
    return _this;
  }

  (0, _createClass2.default)(TooltipView, [{
    key: "didUpdate",
    value: function didUpdate() {
      var props = this.props,
          rootRef = this.rootRef,
          itemsRef = this.itemsRef;
      var group = rootRef.current;

      if (!group) {
        return;
      }

      var records = props.records,
          coord = props.coord;
      if (!records || !records.length) return null;
      var firstRecord = records[0];
      var x = firstRecord.x;
      var coordLeft = coord.left,
          coordWidth = coord.width;

      var _group$get = group.get('attrs'),
          width = _group$get.width;

      var halfWidth = width / 2;
      var moveX = Math.min(Math.max(x - coordLeft - halfWidth, 0), coordWidth - halfWidth);
      itemsRef.current.moveTo(moveX, 0);
    }
  }, {
    key: "render",
    value: function render() {
      var props = this.props;
      var records = props.records,
          chart = props.chart,
          coord = props.coord,
          layout = props.layout;
      var layoutTop = layout.top;
      if (!records || !records.length) return null;
      var coordLeft = coord.left,
          coordTop = coord.top,
          coordRight = coord.right,
          coordBottom = coord.bottom;
      var firstRecord = records[0];
      var x = firstRecord.x,
          y = firstRecord.y,
          xField = firstRecord.xField,
          yField = firstRecord.yField,
          firstOrigin = firstRecord.origin;
      var yScale = chart.getScale(yField);
      var background = props.background,
          showTitle = props.showTitle,
          titleStyle = props.titleStyle,
          _props$showItemMarker = props.showItemMarker,
          showItemMarker = _props$showItemMarker === void 0 ? defaultStyle.showItemMarker : _props$showItemMarker,
          customItemMarkerStyle = props.itemMarkerStyle,
          nameStyle = props.nameStyle,
          valueStyle = props.valueStyle,
          _props$joinString = props.joinString,
          joinString = _props$joinString === void 0 ? defaultStyle.joinString : _props$joinString,
          _props$showCrosshairs = props.showCrosshairs,
          showCrosshairs = _props$showCrosshairs === void 0 ? defaultStyle.showCrosshairs : _props$showCrosshairs,
          crosshairsStyle = props.crosshairsStyle,
          _props$crosshairsType = props.crosshairsType,
          crosshairsType = _props$crosshairsType === void 0 ? defaultStyle.crosshairsType : _props$crosshairsType,
          _props$snap = props.snap,
          snap = _props$snap === void 0 ? defaultStyle.snap : _props$snap;
      var itemMarkerStyle = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, customItemMarkerStyle), defaultStyle.itemMarkerStyle);
      return (0, _jsx.jsx)("group", null, snap ? records.map(function (item) {
        var x = item.x,
            y = item.y,
            color = item.color,
            shape = item.shape;
        return (0, _jsx.jsx)("circle", {
          attrs: (0, _objectSpread2.default)({
            x: x,
            y: y,
            r: 3,
            fill: '#fff',
            lineWidth: '3px',
            stroke: color
          }, shape)
        });
      }) : null, (0, _jsx.jsx)("group", {
        ref: this.rootRef,
        style: {
          left: coordLeft,
          top: layoutTop
        }
      }, (0, _jsx.jsx)("group", {
        ref: this.itemsRef,
        style: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultStyle.background), background),
        attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultStyle.background), background)
      }, showTitle ? (0, _jsx.jsx)("text", {
        style: {
          marginBottom: '6px'
        },
        attrs: (0, _objectSpread2.default)({
          text: firstOrigin[xField],
          fontSize: '24px',
          fill: '#fff',
          textAlign: 'start'
        }, titleStyle)
      }) : null, (0, _jsx.jsx)("group", {
        style: {
          display: 'flex',
          flexDirection: 'row',
          flexWrap: 'wrap'
        }
      }, records.map(function (record) {
        var value = yScale.getText(record[yField]);
        return (0, _jsx.jsx)("group", {
          style: {
            display: 'flex',
            flexDirection: 'row',
            alignItems: 'center',
            padding: [0, '10px', 0, 0]
          }
        }, showItemMarker ? (0, _jsx.jsx)("marker", {
          style: {
            width: itemMarkerStyle.width,
            marginRight: '6px'
          },
          attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)({}, itemMarkerStyle), {}, {
            fill: record.color
          })
        }) : null, (0, _jsx.jsx)("text", {
          attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultStyle.nameStyle), nameStyle), {}, {
            text: value ? "".concat(record[xField]).concat(joinString) : record[xField]
          })
        }), (0, _jsx.jsx)("text", {
          attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultStyle.valueStyle), valueStyle), {}, {
            text: value
          })
        }));
      }))), showCrosshairs ? (0, _jsx.jsx)("group", null, directionEnabled(crosshairsType, 'x') ? (0, _jsx.jsx)("line", {
        attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)({
          x1: coordLeft,
          y1: y,
          x2: coordRight,
          y2: y
        }, defaultStyle.crosshairsStyle), crosshairsStyle)
      }) : null, directionEnabled(crosshairsType, 'y') ? (0, _jsx.jsx)("line", {
        attrs: (0, _objectSpread2.default)((0, _objectSpread2.default)({
          x1: x,
          y1: coordTop,
          x2: x,
          y2: coordBottom
        }, defaultStyle.crosshairsStyle), crosshairsStyle)
      }) : null) : null));
    }
  }]);
  return TooltipView;
}(_component.default);

exports.default = TooltipView;